@@ -28,128 +28,99 @@ short_description: Manages VRRP interfaces on HUAWEI CloudEngine devices.
 description:
     - Manages VRRP interface attributes on HUAWEI CloudEngine devices.
 author:
-    - Li Yanfeng (@CloudEngine-Ansible)
+    - Li Yanfeng (@numone213)
+notes:
+    - This module requires the netconf system service be enabled on the remote device being managed.
+    - Recommended connection is C(netconf).
+    - This module also works with C(local) connections for legacy playbooks.
 options:
     interface:
         description:
             - Name of an interface. The value is a string of 1 to 63 characters.
-        required: false
-        default: null
     vrid:
         description:
             - VRRP backup group ID.
               The value is an integer ranging from 1 to 255.
-        required: false
         default: present
     virtual_ip :
         description:
             - Virtual IP address. The value is a string of 0 to 255 characters.
-        required: false
-        default: null
     vrrp_type:
         description:
             - Type of a VRRP backup group.
-        required: false
         choices: ['normal', 'member', 'admin']
-        default: null
     admin_ignore_if_down:
         description:
             - mVRRP ignores an interface Down event.
-        required: false
-        default: False
+        type: bool
+        default: 'false'
     admin_vrid:
         description:
             - Tracked mVRRP ID. The value is an integer ranging from 1 to 255.
-        required: false
-        default: null
     admin_interface:
         description:
             - Tracked mVRRP interface name. The value is a string of 1 to 63 characters.
-        required: false
-        default: null
     admin_flowdown:
         description:
             - Disable the flowdown function for service VRRP.
-        required: false
-        default: False
+        type: bool
+        default: 'false'
     priority:
         description:
             - Configured VRRP priority.
               The value ranges from 1 to 254. The default value is 100. A larger value indicates a higher priority.
-        required: false
-        default: null
     version:
         description:
             - VRRP version. The default version is v2.
-        required: false
         choices: ['v2','v3']
-        default: null
     advertise_interval:
         description:
             - Configured interval between sending advertisements, in milliseconds.
               Only the master router sends VRRP advertisements. The default value is 1000 milliseconds.
-        required: false
-        default: null
     preempt_timer_delay:
         description:
             - Preemption delay.
               The value is an integer ranging from 0 to 3600. The default value is 0.
-        required: false
-        default: null
     gratuitous_arp_interval:
         description:
             - Interval at which gratuitous ARP packets are sent, in seconds.
               The value ranges from 30 to 1200.The default value is 300.
-        required: false
-        default: null
     recover_delay:
         description:
             - Delay in recovering after an interface goes Up.
               The delay is used for interface flapping suppression.
               The value is an integer ranging from 0 to 3600.
               The default value is 0 seconds.
-        required: false
-        default: null
     holding_multiplier:
         description:
             - The configured holdMultiplier.The value is an integer ranging from 3 to 10. The default value is 3.
-        required: false
-        default: null
     auth_mode:
         description:
             - Authentication type used for VRRP packet exchanges between virtual routers.
               The values are noAuthentication, simpleTextPassword, md5Authentication.
               The default value is noAuthentication.
-        required: false
         choices: ['simple','md5','none']
-        default: null
     is_plain:
         description:
             - Select the display mode of an authentication key.
               By default, an authentication key is displayed in ciphertext.
-        required: false
-        default: False
+        type: bool
+        default: 'false'
     auth_key:
         description:
             - This object is set based on the authentication type.
               When noAuthentication is specified, the value is empty.
               When simpleTextPassword or md5Authentication is specified, the value is a string of 1 to 8 characters
               in plaintext and displayed as a blank text for security.
-        required: false
-        default: null
     fast_resume:
         description:
             - mVRRP's fast resume mode.
-        required: false
         choices: ['enable','disable']
-        default: null
     state:
         description:
             - Specify desired state of the resource.
-        required: false
         default: present
         choices: ['present','absent']
-
 '''
 
 EXAMPLES = '''
@@ -164,60 +135,50 @@ EXAMPLES = '''
       username: "{{ username }}"
       password: "{{ password }}"
       transport: cli
-
   tasks:
-
   - name: Set vrrp version
     ce_vrrp:
       version: v3
       provider: "{{ cli }}"
-
   - name: Set vrrp gratuitous-arp interval
     ce_vrrp:
       gratuitous_arp_interval: 40
       mlag_id: 4
       provider: "{{ cli }}"
-
   - name: Set vrrp recover-delay
     ce_vrrp:
       recover_delay: 10
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid virtual-ip
     ce_vrrp:
       interface: 40GE2/0/8
       vrid: 1
       virtual_ip: 10.14.2.7
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid admin
     ce_vrrp:
       interface: 40GE2/0/8
       vrid: 1
       vrrp_type: admin
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid fast_resume
     ce_vrrp:
       interface: 40GE2/0/8
       vrid: 1
       fast_resume: enable
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid holding-multiplier
     ce_vrrp:
       interface: 40GE2/0/8
       vrid: 1
       holding_multiplier: 4
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid preempt timer delay
     ce_vrrp:
       interface: 40GE2/0/8
       vrid: 1
       preempt_timer_delay: 10
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid admin-vrrp
     ce_vrrp:
       interface: 40GE2/0/8
@@ -226,7 +187,6 @@ EXAMPLES = '''
       admin_vrid: 2
       vrrp_type: member
       provider: "{{ cli }}"
-
   - name: Set vrrp vrid authentication-mode
     ce_vrrp:
       interface: 40GE2/0/8
@@ -241,7 +201,7 @@ RETURN = '''
 changed:
     description: check to see if a change was made on the device
     returned: always
-    type: boolean
+    type: bool
     sample: true
 proposed:
     description: k/v pairs of parameters passed into module
@@ -310,14 +270,12 @@ CE_NC_SET_VRRP_GROUP_INFO_HEAD = """
       <vrrpGroup operation="merge">
         <ifName>%s</ifName>
         <vrrpId>%s</vrrpId>
-
 """
 CE_NC_SET_VRRP_GROUP_INFO_TAIL = """
       </vrrpGroup>
     </vrrpGroups>
   </vrrp>
 </config>
-
 """
 CE_NC_GET_VRRP_GLOBAL_INFO = """
 <filter type="subtree">
@@ -330,7 +288,6 @@ CE_NC_GET_VRRP_GLOBAL_INFO = """
     </vrrpGlobalCfg>
   </vrrp>
 </filter>
-
 """
 
 CE_NC_SET_VRRP_GLOBAL_HEAD = """
@@ -342,7 +299,6 @@ CE_NC_SET_VRRP_GLOBAL_TAIL = """
     </vrrpGlobalCfg>
   </vrrp>
 </config>
-
 """
 
 CE_NC_GET_VRRP_VIRTUAL_IP_INFO = """
@@ -361,7 +317,6 @@ CE_NC_GET_VRRP_VIRTUAL_IP_INFO = """
     </vrrpGroups>
   </vrrp>
 </filter>
-
 """
 CE_NC_CREATE_VRRP_VIRTUAL_IP_INFO = """
 <config>
@@ -379,7 +334,6 @@ CE_NC_CREATE_VRRP_VIRTUAL_IP_INFO = """
     </vrrpGroups>
   </vrrp>
 </config>
-
 """
 CE_NC_DELETE_VRRP_VIRTUAL_IP_INFO = """
 <config>
@@ -397,7 +351,6 @@ CE_NC_DELETE_VRRP_VIRTUAL_IP_INFO = """
     </vrrpGroups>
   </vrrp>
 </config>
-
 """
 
 
@@ -463,10 +416,10 @@ class Vrrp(object):
         self.vrid = self.module.params['vrid']
         self.virtual_ip = self.module.params['virtual_ip']
         self.vrrp_type = self.module.params['vrrp_type']
-        self.admin_ignore_if_down = self.module.params['admin_ignore_if_down']
+        self.admin_ignore_if_down = 'false' if self.module.params['admin_ignore_if_down'] is False else 'true'
         self.admin_vrid = self.module.params['admin_vrid']
         self.admin_interface = self.module.params['admin_interface']
-        self.admin_flowdown = self.module.params['admin_flowdown']
+        self.admin_flowdown = 'false' if self.module.params['admin_flowdown'] is False else 'true'
         self.priority = self.module.params['priority']
         self.version = self.module.params['version']
         self.advertise_interval = self.module.params['advertise_interval']
@@ -476,7 +429,7 @@ class Vrrp(object):
         self.recover_delay = self.module.params['recover_delay']
         self.holding_multiplier = self.module.params['holding_multiplier']
         self.auth_mode = self.module.params['auth_mode']
-        self.is_plain = self.module.params['is_plain']
+        self.is_plain = 'false' if self.module.params['is_plain'] is False else 'true'
         self.auth_key = self.module.params['auth_key']
         self.fast_resume = self.module.params['fast_resume']
         self.state = self.module.params['state']
@@ -514,7 +467,7 @@ class Vrrp(object):
             virtual_ip_info["vrrpVirtualIpInfos"] = list()
             root = ElementTree.fromstring(xml_str)
             vrrp_virtual_ip_infos = root.findall(
-                "data/vrrp/vrrpGroups/vrrpGroup/virtualIps/virtualIp")
+                "vrrp/vrrpGroups/vrrpGroup/virtualIps/virtualIp")
             if vrrp_virtual_ip_infos:
                 for vrrp_virtual_ip_info in vrrp_virtual_ip_infos:
                     virtual_ip_dict = dict()
@@ -540,7 +493,7 @@ class Vrrp(object):
 
             root = ElementTree.fromstring(xml_str)
             global_info = root.findall(
-                "data/vrrp/vrrpGlobalCfg")
+                "vrrp/vrrpGlobalCfg")
 
             if global_info:
                 for tmp in global_info:
@@ -564,7 +517,7 @@ class Vrrp(object):
 
             root = ElementTree.fromstring(xml_str)
             global_info = root.findall(
-                "data/vrrp/vrrpGroups/vrrpGroup")
+                "vrrp/vrrpGroups/vrrpGroup")
 
             if global_info:
                 for tmp in global_info:
@@ -633,10 +586,12 @@ class Vrrp(object):
             if not self.advertise_interval.isdigit():
                 self.module.fail_json(
                     msg='Error: The value of advertise_interval is an integer.')
-            if int(self.advertise_interval) < 1 or int(self.advertise_interval) > 255000:
+            if int(self.advertise_interval) < 1000 or int(self.advertise_interval) > 255000:
                 self.module.fail_json(
-                    msg='Error: The value of advertise_interval ranges from 1 to 255000. The default value is 1000.')
-
+                    msg='Error: The value of advertise_interval ranges from 1000 to 255000 milliseconds. The default value is 1000 milliseconds.')
+            if int(self.advertise_interval) % 1000 != 0:
+                self.module.fail_json(
+                    msg='Error: The advertisement interval value of VRRP must be a multiple of 1000 milliseconds.')
         # preempt_timer_delay check
         if self.preempt_timer_delay:
             if not self.preempt_timer_delay.isdigit():
@@ -725,12 +680,11 @@ class Vrrp(object):
 
     def is_vrrp_group_info_change(self):
         """whether vrrp group attribute info change"""
-
         if self.vrrp_type:
             if self.vrrp_group_info["vrrpType"] != self.vrrp_type:
                 return True
         if self.admin_ignore_if_down:
-            if self.vrrp_group_info["adminIgnoreIfDown"] != self.admin_ignore_if_down:
+            if self.vrrp_group_info["adminIgnoreIfDown"] != str(self.admin_ignore_if_down).lower():
                 return True
         if self.admin_vrid:
             if self.vrrp_group_info["adminVrrpId"] != self.admin_vrid:
@@ -777,7 +731,7 @@ class Vrrp(object):
             if self.vrrp_group_info["vrrpType"] != self.vrrp_type:
                 return False
         if self.admin_ignore_if_down:
-            if self.vrrp_group_info["adminIgnoreIfDown"] != self.admin_ignore_if_down:
+            if self.vrrp_group_info["adminIgnoreIfDown"] != str(self.admin_ignore_if_down).lower():
                 return False
         if self.admin_vrid:
             if self.vrrp_group_info["adminVrrpId"] != self.admin_vrid:
@@ -861,7 +815,7 @@ class Vrrp(object):
             recv_xml = set_nc_config(self.module, conf_str)
             if "<ok/>" not in recv_xml:
                 self.module.fail_json(
-                    msg='Error: set vrrp global atrribute info failed.')
+                    msg='Error: set vrrp global attribute info failed.')
 
             if self.gratuitous_arp_interval:
                 self.updates_cmd.append(
@@ -905,7 +859,7 @@ class Vrrp(object):
             recv_xml = set_nc_config(self.module, conf_str)
             if "<ok/>" not in recv_xml:
                 self.module.fail_json(
-                    msg='Error: set vrrp global atrribute info failed.')
+                    msg='Error: set vrrp global attribute info failed.')
             if self.gratuitous_arp_interval:
                 self.updates_cmd.append("undo vrrp gratuitous-arp interval")
 
@@ -966,77 +920,58 @@ class Vrrp(object):
             recv_xml = set_nc_config(self.module, conf_str)
             if "<ok/>" not in recv_xml:
                 self.module.fail_json(
-                    msg='Error: set vrrp group atrribute info failed.')
-
+                    msg='Error: set vrrp group attribute info failed.')
             if self.interface and self.vrid:
+                self.updates_cmd.append("interface %s" % self.interface)
                 if self.vrrp_type == "admin":
                     if self.admin_ignore_if_down is True:
-                        self.updates_cmd.append(
-                            "interface %s" % self.interface)
                         self.updates_cmd.append(
                             "vrrp vrid %s admin ignore-if-down" % self.vrid)
                     else:
-                        self.updates_cmd.append(
-                            "interface %s" % self.interface)
                         self.updates_cmd.append(
                             "vrrp vrid %s admin" % self.vrid)
 
                 if self.priority:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "vrrp vrid %s priority %s" % (self.vrid, self.priority))
 
                 if self.fast_resume == "enable":
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "vrrp vrid %s fast-resume" % self.vrid)
                 if self.fast_resume == "disable":
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s fast-resume" % self.vrid)
 
                 if self.advertise_interval:
-                    self.updates_cmd.append("interface %s" % self.interface)
-                    self.updates_cmd.append("vrrp vrid %s timer advertise %s" % (
-                        self.vrid, self.advertise_interval))
+                    advertise_interval = int(self.advertise_interval) / 1000
+                    self.updates_cmd.append("vrrp vrid %s timer advertise %s<seconds>" % (
+                        self.vrid, int(advertise_interval)))
 
                 if self.preempt_timer_delay:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append("vrrp vrid %s preempt timer delay %s" % (self.vrid,
                                                                                      self.preempt_timer_delay))
 
                 if self.holding_multiplier:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "vrrp vrid %s holding-multiplier %s" % (self.vrid, self.holding_multiplier))
 
                 if self.admin_vrid and self.admin_interface:
-                    if self.admin_flowdown is True:
-                        self.updates_cmd.append(
-                            "interface %s" % self.interface)
+                    if self.admin_flowdown == "true":
                         self.updates_cmd.append("vrrp vrid %s track admin-vrrp interface %s vrid %s unflowdown" %
                                                 (self.vrid, self.admin_interface, self.admin_vrid))
                     else:
-                        self.updates_cmd.append(
-                            "interface %s" % self.interface)
                         self.updates_cmd.append("vrrp vrid %s track admin-vrrp interface %s vrid %s" %
                                                 (self.vrid, self.admin_interface, self.admin_vrid))
 
                 if self.auth_mode and self.auth_key:
                     if self.auth_mode == "simple":
-                        if self.is_plain is True:
-                            self.updates_cmd.append(
-                                "interface %s" % self.interface)
+                        if self.is_plain == "true":
                             self.updates_cmd.append("vrrp vrid %s authentication-mode simple plain %s" %
                                                     (self.vrid, self.auth_key))
                         else:
-                            self.updates_cmd.append(
-                                "interface %s" % self.interface)
                             self.updates_cmd.append("vrrp vrid %s authentication-mode simple cipher %s" %
                                                     (self.vrid, self.auth_key))
                     if self.auth_mode == "md5":
-                        self.updates_cmd.append(
-                            "interface %s" % self.interface)
                         self.updates_cmd.append(
                             "vrrp vrid %s authentication-mode md5 %s" % (self.vrid, self.auth_key))
                 self.changed = True
@@ -1094,44 +1029,38 @@ class Vrrp(object):
             recv_xml = set_nc_config(self.module, conf_str)
             if "<ok/>" not in recv_xml:
                 self.module.fail_json(
-                    msg='Error: set vrrp global atrribute info failed.')
+                    msg='Error: set vrrp global attribute info failed.')
             if self.interface and self.vrid:
+                self.updates_cmd.append("interface %s" % self.interface)
                 if self.vrrp_type == "admin":
                     self.updates_cmd.append(
                         "undo vrrp vrid %s admin" % self.vrid)
 
                 if self.priority:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s priority" % self.vrid)
 
                 if self.fast_resume:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s fast-resume" % self.vrid)
 
                 if self.advertise_interval:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s timer advertise" % self.vrid)
 
                 if self.preempt_timer_delay:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s preempt timer delay" % self.vrid)
 
                 if self.holding_multiplier:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s holding-multiplier" % self.vrid)
 
                 if self.admin_vrid and self.admin_interface:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s track admin-vrrp" % self.vrid)
 
                 if self.auth_mode:
-                    self.updates_cmd.append("interface %s" % self.interface)
                     self.updates_cmd.append(
                         "undo vrrp vrid %s authentication-mode" % self.vrid)
                 self.changed = True
@@ -1206,7 +1135,7 @@ class Vrrp(object):
             self.existing["vrrp_type"] = self.vrrp_group_info["vrrpType"]
             if self.vrrp_type == "admin":
                 self.existing["admin_ignore_if_down"] = self.vrrp_group_info[
-                    "authenticationMode"]
+                    "adminIgnoreIfDown"]
             if self.admin_vrid and self.admin_interface:
                 self.existing["admin_vrid"] = self.vrrp_group_info[
                     "adminVrrpId"]
@@ -1269,7 +1198,7 @@ class Vrrp(object):
             self.end_state["vrrp_type"] = self.vrrp_group_info["vrrpType"]
             if self.vrrp_type == "admin":
                 self.end_state["admin_ignore_if_down"] = self.vrrp_group_info[
-                    "authenticationMode"]
+                    "adminIgnoreIfDown"]
             if self.admin_vrid and self.admin_interface:
                 self.existing["admin_vrid"] = self.vrrp_group_info[
                     "adminVrrpId"]
@@ -1344,7 +1273,7 @@ class Vrrp(object):
                         self.module.fail_json(
                             msg='Error: admin_interface admin_vrid vrrp_type interface vrid must '
                                 'be config at the same time.')
-                if self.auth_mode == "md5" and self.is_plain is True:
+                if self.auth_mode == "md5" and self.is_plain == "true":
                     self.module.fail_json(
                         msg='Error: is_plain can not be True when auth_mode is md5.')
 
