@@ -8,37 +8,41 @@ __metaclass__ = type
 
 import os
 import json
-import sys
 import pytest
+import sys
 
-from nose.plugins.skip import SkipTest
 if sys.version_info < (2, 7):
-    raise SkipTest("F5 Ansible modules require Python >= 2.7")
+    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")
 
-from ansible.compat.tests import unittest
-from ansible.compat.tests.mock import Mock
-from ansible.compat.tests.mock import patch
 from ansible.module_utils.basic import AnsibleModule
 
 try:
-    from library.bigip_monitor_tcp_half_open import Parameters
-    from library.bigip_monitor_tcp_half_open import ModuleManager
-    from library.bigip_monitor_tcp_half_open import ArgumentSpec
-    from library.bigip_monitor_tcp_half_open import HAS_F5SDK
+    from library.modules.bigip_monitor_tcp_half_open import Parameters
+    from library.modules.bigip_monitor_tcp_half_open import ModuleManager
+    from library.modules.bigip_monitor_tcp_half_open import ArgumentSpec
+
     from library.module_utils.network.f5.common import F5ModuleError
-    from library.module_utils.network.f5.common import iControlUnexpectedHTTPError
-    from test.unit.modules.utils import set_module_args
+
+    # In Ansible 2.8, Ansible changed import paths.
+    from test.units.compat import unittest
+    from test.units.compat.mock import Mock
+    from test.units.compat.mock import patch
+
+    from test.units.modules.utils import set_module_args
 except ImportError:
-    try:
-        from ansible.modules.network.f5.bigip_monitor_tcp_half_open import Parameters
-        from ansible.modules.network.f5.bigip_monitor_tcp_half_open import ModuleManager
-        from ansible.modules.network.f5.bigip_monitor_tcp_half_open import ArgumentSpec
-        from ansible.modules.network.f5.bigip_monitor_tcp_half_open import HAS_F5SDK
-        from ansible.module_utils.network.f5.common import F5ModuleError
-        from ansible.module_utils.network.f5.common import iControlUnexpectedHTTPError
-        from units.modules.utils import set_module_args
-    except ImportError:
-        raise SkipTest("F5 Ansible modules require the f5-sdk Python library")
+    from ansible.modules.network.f5.bigip_monitor_tcp_half_open import Parameters
+    from ansible.modules.network.f5.bigip_monitor_tcp_half_open import ModuleManager
+    from ansible.modules.network.f5.bigip_monitor_tcp_half_open import ArgumentSpec
+
+    from ansible.module_utils.network.f5.common import F5ModuleError
+
+    # Ansible 2.8 imports
+    from units.compat import unittest
+    from units.compat.mock import Mock
+    from units.compat.mock import patch
+
+    from units.modules.utils import set_module_args
+
 
 fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
 fixture_data = {}
@@ -144,9 +148,11 @@ class TestManager(unittest.TestCase):
             interval=20,
             timeout=30,
             time_until_up=60,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         module = AnsibleModule(
@@ -171,9 +177,11 @@ class TestManager(unittest.TestCase):
             interval=20,
             timeout=30,
             time_until_up=60,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
@@ -195,9 +203,11 @@ class TestManager(unittest.TestCase):
         set_module_args(dict(
             name='foo',
             interval=10,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
@@ -221,9 +231,11 @@ class TestManager(unittest.TestCase):
         set_module_args(dict(
             name='foo',
             interval=30,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
@@ -241,16 +253,18 @@ class TestManager(unittest.TestCase):
         with pytest.raises(F5ModuleError) as ex:
             mm.exec_module()
 
-        assert "must be less than" in str(ex)
+        assert "must be less than" in str(ex.value)
 
     def test_update_interval_larger_than_new_timeout(self, *args):
         set_module_args(dict(
             name='foo',
             interval=10,
             timeout=5,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
@@ -268,15 +282,17 @@ class TestManager(unittest.TestCase):
         with pytest.raises(F5ModuleError) as ex:
             mm.exec_module()
 
-        assert "must be less than" in str(ex)
+        assert "must be less than" in str(ex.value)
 
     def test_update_timeout(self, *args):
         set_module_args(dict(
             name='foo',
             timeout=300,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
@@ -299,9 +315,11 @@ class TestManager(unittest.TestCase):
         set_module_args(dict(
             name='foo',
             time_until_up=300,
-            server='localhost',
-            password='password',
-            user='admin'
+            provider=dict(
+                server='localhost',
+                password='password',
+                user='admin'
+            )
         ))
 
         current = Parameters(params=load_fixture('load_ltm_monitor_tcp_half_open.json'))
